using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using System.Linq;
using Microsoft.Extensions.Logging;
using TransmissionRssManager.Core;

namespace TransmissionRssManager.Services
{
    public class TransmissionClient : ITransmissionClient
    {
        private readonly ILogger<TransmissionClient> _logger;
        private readonly IConfigService _configService;
        private readonly HttpClient _httpClient;
        private string _sessionId = string.Empty;

        public TransmissionClient(ILogger<TransmissionClient> logger, IConfigService configService)
        {
            _logger = logger;
            _configService = configService;
            
            // Configure the main HttpClient with handler that ignores certificate errors
            var handler = new HttpClientHandler
            {
                ServerCertificateCustomValidationCallback = (sender, cert, chain, sslPolicyErrors) => true
            };
            _httpClient = new HttpClient(handler);
            _httpClient.Timeout = TimeSpan.FromSeconds(10);
            
            _logger.LogInformation("TransmissionClient initialized with certificate validation disabled");
        }

        public async Task<List<TorrentInfo>> GetTorrentsAsync()
        {
            var config = _configService.GetConfiguration();
            var request = new
            {
                method = "torrent-get",
                arguments = new
                {
                    fields = new[] { "id", "name", "status", "percentDone", "totalSize", "downloadDir" }
                }
            };

            var response = await SendRequestAsync<TorrentGetResponse>(config.Transmission.Url, request);
            
            _logger.LogInformation($"Transmission torrent response: {response != null}, Arguments: {response?.Arguments != null}, Result: {response?.Result}");
            
            if (response?.Arguments?.Torrents == null)
            {
                _logger.LogWarning("No torrents found in response");
                return new List<TorrentInfo>();
            }
            
            _logger.LogInformation($"Found {response.Arguments.Torrents.Count} torrents in response");

            var torrents = new List<TorrentInfo>();
            foreach (var torrent in response.Arguments.Torrents)
            {
                _logger.LogInformation($"Processing torrent: {torrent.Id} - {torrent.Name}");
                torrents.Add(new TorrentInfo
                {
                    Id = torrent.Id,
                    Name = torrent.Name,
                    Status = GetStatusText(torrent.Status),
                    PercentDone = torrent.PercentDone,
                    TotalSize = torrent.TotalSize,
                    DownloadDir = torrent.DownloadDir
                });
            }

            return torrents;
        }

        public async Task<int> AddTorrentAsync(string torrentUrl, string downloadDir)
        {
            var config = _configService.GetConfiguration();
            var request = new
            {
                method = "torrent-add",
                arguments = new
                {
                    filename = torrentUrl,
                    downloadDir = downloadDir
                }
            };

            var response = await SendRequestAsync<TorrentAddResponse>(config.Transmission.Url, request);
            
            if (response?.Arguments?.TorrentAdded != null)
            {
                return response.Arguments.TorrentAdded.Id;
            }
            else if (response?.Arguments?.TorrentDuplicate != null)
            {
                return response.Arguments.TorrentDuplicate.Id;
            }
            
            throw new Exception("Failed to add torrent");
        }

        public async Task RemoveTorrentAsync(int id, bool deleteLocalData)
        {
            var config = _configService.GetConfiguration();
            var request = new
            {
                method = "torrent-remove",
                arguments = new
                {
                    ids = new[] { id },
                    deleteLocalData = deleteLocalData
                }
            };

            await SendRequestAsync<object>(config.Transmission.Url, request);
        }

        public async Task StartTorrentAsync(int id)
        {
            var config = _configService.GetConfiguration();
            var request = new
            {
                method = "torrent-start",
                arguments = new
                {
                    ids = new[] { id }
                }
            };

            await SendRequestAsync<object>(config.Transmission.Url, request);
        }

        public async Task StopTorrentAsync(int id)
        {
            var config = _configService.GetConfiguration();
            var request = new
            {
                method = "torrent-stop",
                arguments = new
                {
                    ids = new[] { id }
                }
            };

            await SendRequestAsync<object>(config.Transmission.Url, request);
        }

        private async Task<T> SendRequestAsync<T>(string url, object requestData)
        {
            var config = _configService.GetConfiguration();
            var jsonContent = JsonSerializer.Serialize(requestData);
            var content = new StringContent(jsonContent, Encoding.UTF8, "application/json");
            
            // Always create a fresh HttpClient to avoid connection issues
            using var httpClient = new HttpClient(new HttpClientHandler 
            { 
                ServerCertificateCustomValidationCallback = (sender, cert, chain, sslPolicyErrors) => true 
            });

            // Ensure we have a valid URL by reconstructing it explicitly
            var protocol = config.Transmission.UseHttps ? "https" : "http";
            var serverUrl = $"{protocol}://{config.Transmission.Host}:{config.Transmission.Port}/transmission/rpc";
            
            var request = new HttpRequestMessage(HttpMethod.Post, serverUrl)
            {
                Content = content
            };

            // Add session ID if we have one
            if (!string.IsNullOrEmpty(_sessionId))
            {
                request.Headers.Add("X-Transmission-Session-Id", _sessionId);
            }

            // Add authentication if provided
            if (!string.IsNullOrEmpty(config.Transmission.Username) && !string.IsNullOrEmpty(config.Transmission.Password))
            {
                var credentials = Convert.ToBase64String(Encoding.ASCII.GetBytes($"{config.Transmission.Username}:{config.Transmission.Password}"));
                request.Headers.Authorization = new AuthenticationHeaderValue("Basic", credentials);
            }

            try
            {
                // Set timeout to avoid hanging indefinitely on connection issues
                httpClient.Timeout = TimeSpan.FromSeconds(10);
                
                _logger.LogInformation($"Connecting to Transmission at {serverUrl} with auth: {!string.IsNullOrEmpty(config.Transmission.Username)}");
                var response = await httpClient.SendAsync(request);
                
                // Check if we need a new session ID
                if (response.StatusCode == System.Net.HttpStatusCode.Conflict)
                {
                    if (response.Headers.TryGetValues("X-Transmission-Session-Id", out var sessionIds))
                    {
                        _sessionId = sessionIds.FirstOrDefault() ?? string.Empty;
                        _logger.LogInformation($"Got new Transmission session ID: {_sessionId}");
                        
                        // Retry request with new session ID
                        return await SendRequestAsync<T>(url, requestData);
                    }
                }
                
                response.EnsureSuccessStatusCode();
                
                var resultContent = await response.Content.ReadAsStringAsync();
                _logger.LogInformation($"Received successful response from Transmission: {resultContent.Substring(0, Math.Min(resultContent.Length, 500))}");
                
                // Configure JSON deserializer to be case insensitive
                var options = new JsonSerializerOptions
                {
                    PropertyNameCaseInsensitive = true
                };
                
                return JsonSerializer.Deserialize<T>(resultContent, options);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, $"Error communicating with Transmission at {serverUrl}: {ex.Message}");
                throw new Exception($"Failed to connect to Transmission at {config.Transmission.Host}:{config.Transmission.Port}. Error: {ex.Message}", ex);
            }
        }

        private string GetStatusText(int status)
        {
            return status switch
            {
                0 => "Stopped",
                1 => "Queued",
                2 => "Verifying",
                3 => "Downloading",
                4 => "Seeding",
                5 => "Queued",
                6 => "Checking",
                _ => "Unknown"
            };
        }

        // Transmission response classes with proper JSON attribute names
        private class TorrentGetResponse
        {
            [System.Text.Json.Serialization.JsonPropertyName("arguments")]
            public TorrentGetArguments Arguments { get; set; }
            
            [System.Text.Json.Serialization.JsonPropertyName("result")]
            public string Result { get; set; }
        }

        private class TorrentGetArguments
        {
            [System.Text.Json.Serialization.JsonPropertyName("torrents")]
            public List<TransmissionTorrent> Torrents { get; set; }
        }

        private class TransmissionTorrent
        {
            [System.Text.Json.Serialization.JsonPropertyName("id")]
            public int Id { get; set; }
            
            [System.Text.Json.Serialization.JsonPropertyName("name")]
            public string Name { get; set; }
            
            [System.Text.Json.Serialization.JsonPropertyName("status")]
            public int Status { get; set; }
            
            [System.Text.Json.Serialization.JsonPropertyName("percentDone")]
            public double PercentDone { get; set; }
            
            [System.Text.Json.Serialization.JsonPropertyName("totalSize")]
            public long TotalSize { get; set; }
            
            [System.Text.Json.Serialization.JsonPropertyName("downloadDir")]
            public string DownloadDir { get; set; }
        }

        private class TorrentAddResponse
        {
            [System.Text.Json.Serialization.JsonPropertyName("arguments")]
            public TorrentAddArguments Arguments { get; set; }
            
            [System.Text.Json.Serialization.JsonPropertyName("result")]
            public string Result { get; set; }
        }

        private class TorrentAddArguments
        {
            [System.Text.Json.Serialization.JsonPropertyName("torrent-added")]
            public TorrentAddInfo TorrentAdded { get; set; }
            
            [System.Text.Json.Serialization.JsonPropertyName("torrent-duplicate")]
            public TorrentAddInfo TorrentDuplicate { get; set; }
        }

        private class TorrentAddInfo
        {
            [System.Text.Json.Serialization.JsonPropertyName("id")]
            public int Id { get; set; }
            
            [System.Text.Json.Serialization.JsonPropertyName("name")]
            public string Name { get; set; }
            
            [System.Text.Json.Serialization.JsonPropertyName("hashString")]
            public string HashString { get; set; }
        }
    }
}